////////////////////////////////////////////////////////////////////////////////////////////////
/*! \file    cvt_V1495.h
*   \brief   V1495 VME board definitions
*   \author  NDA
*   \version 1.0
*   \date    03/2006
*            
*            Provides methods, properties and defines to handle with V1495 VME boards
*/
////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef __CVT_V1495_DEF_H
#define __CVT_V1495_DEF_H

////////////////////////////////////////////////////////////////////////////////////////////////
// File includes
////////////////////////////////////////////////////////////////////////////////////////////////
#include "cvt_common_defs.h"
#include "cvt_board_commons.h"

////////////////////////////////////////////////////////////////////////////////////////////////
// Global defines
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \struct  cvt_V1495_data
*   \brief   V1495 board data structure
*            
*            This structure stores any board data, common and specific ones. 
*            It must be passed to each V1495's API 
*/
////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct
{
	cvt_board_data	m_common_data;				//!< Boards common data structure: always let at offset 0
	//
	// Board specific data

} cvt_V1495_data;


////////////////////////////////////////////////////////////////////////////////////////////////
// Registers address
////////////////////////////////////////////////////////////////////////////////////////////////
#define CVT_V1495_CONTROL_ADD				0x0002					/*!< \brief Control register relative address */
#define CVT_V1495_PORTA_EN_LOW_ADD			0x0010					/*!< \brief Port A enable low word relative address */
#define CVT_V1495_PORTA_EN_HIGH_ADD			0x0012					/*!< \brief Port A enable high word relative address */
#define CVT_V1495_PORTB_EN_LOW_ADD			0x0014					/*!< \brief Port B enable low word relative address */
#define CVT_V1495_PORTB_EN_HIGH_ADD			0x0016					/*!< \brief Port B enable high word relative address */
#define CVT_V1495_PORTD_EN_LOW_ADD			0x0018					/*!< \brief Port D enable low word relative address */
#define CVT_V1495_PORTD_EN_HIGH_ADD			0x001A					/*!< \brief Port D enable high word relative address */

////////////////////////////////////////////////////////////////////////////////////////////////
// Registers data size
////////////////////////////////////////////////////////////////////////////////////////////////
#define CVT_V1495_CONTROL_DATA_SIZE			cvD16					/*!< \brief Control register data size */
#define CVT_V1495_PORTA_EN_LOW_DATA_SIZE	cvD16					/*!< \brief Port A enable low word data size */
#define CVT_V1495_PORTA_EN_HIGH_DATA_SIZE	cvD16					/*!< \brief Port A enable high word data size */
#define CVT_V1495_PORTB_EN_LOW_DATA_SIZE	cvD16					/*!< \brief Port B enable low word data size */
#define CVT_V1495_PORTB_EN_HIGH_DATA_SIZE	cvD16					/*!< \brief Port B enable high word data size */
#define CVT_V1495_PORTD_EN_LOW_DATA_SIZE	cvD16					/*!< \brief Port D enable low word data size */
#define CVT_V1495_PORTD_EN_HIGH_DATA_SIZE	cvD16					/*!< \brief Port D enable high word data size */

////////////////////////////////////////////////////////////////////////////////////////////////
// Registers address modifiers
////////////////////////////////////////////////////////////////////////////////////////////////
#define CVT_V1495_CONTROL_AM				cvA32_S_DATA			/*!< \brief Control register address modifier */
#define CVT_V1495_PORTA_EN_LOW_AM			cvA32_S_DATA			/*!< \brief Port A enable low word address modifier */
#define CVT_V1495_PORTA_EN_HIGH_AM			cvA32_S_DATA			/*!< \brief Port A enable high word address modifier */
#define CVT_V1495_PORTB_EN_LOW_AM			cvA32_S_DATA			/*!< \brief Port B enable low word address modifier */
#define CVT_V1495_PORTB_EN_HIGH_AM			cvA32_S_DATA			/*!< \brief Port B enable high word address modifier */
#define CVT_V1495_PORTD_EN_LOW_AM			cvA32_S_DATA			/*!< \brief Port D enable low word address modifier */
#define CVT_V1495_PORTD_EN_HIGH_AM			cvA32_S_DATA			/*!< \brief Port D enable high word address modifier */

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \enum    CVT_V1495_REG_INDEX
*   \brief   The registers indexes
*            
*            Provides an entry for each register: This is the index into the CVT_V1495_REG_TABLE board table
*/
////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum
{
	CVT_V1495_CONTROL_INDEX,			/*!< \brief Control register index */
	CVT_V1495_PORTA_EN_LOW_INDEX,		/*!< \brief Port A enable low word index */
	CVT_V1495_PORTA_EN_HIGH_INDEX,		/*!< \brief Port A enable high word index */
	CVT_V1495_PORTB_EN_LOW_INDEX,		/*!< \brief Port B enable low word index */
	CVT_V1495_PORTB_EN_HIGH_INDEX,		/*!< \brief Port B enable high word index */
	CVT_V1495_PORTD_EN_LOW_INDEX,		/*!< \brief Port D enable low word index */
	CVT_V1495_PORTD_EN_HIGH_INDEX,		/*!< \brief Port D enable high word index */
} CVT_V1495_REG_INDEX;

////////////////////////////////////////////////////////////////////////////////////////////////
// Global variables declaration
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
// Global methods declaration
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
//
//     B O A R D S   H A N D L I N G
//
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1495_open( cvt_V1495_data* p_data, UINT16 base_address, long vme_handle);
*   \brief   V1494 VME boards data initialization
*            
*            Provides specific handling for V495 boards opening.
*   \param   p_data Pointer to board data
*   \param   base_address The board base address (MSW)
*   \param   vme_handle The VME handle
*   \return  TRUE: board successfully opened
*   \note    Must be called before any other board specific API.
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1495_open( cvt_V1495_data* p_data, UINT16 base_address, long vme_handle);

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1495_close( cvt_V1495_data* p_data);
*   \brief   V1495 VME boards closing and resource free
*            
*            Provides specific handling for V1495 boards closing.
*   \param   p_data Pointer to board data
*   \return  TRUE: board successfully closed
*   \note    Must be called when done with any other board specific API.
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1495_close( cvt_V1495_data* p_data);

////////////////////////////////////////////////////////////////////////////////////////////////
//
//     L E V E L   0   A P I s
//
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
//
//     L E V E L   1   A P I s
//
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1495_set_control( cvt_V1495_data* p_data, UINT16 value);
*   \brief   Set the control register.
*            
*            Writes the specified value to the control register
*   \param   p_data Pointer to board data
*   \param   value The value to set
*   \return  TRUE: Procedure successfully executed
*   \sa      cvt_V1495_threshold_id 
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1495_set_control( cvt_V1495_data* p_data, UINT16 value);

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1495_set_port_A_enable( cvt_V1495_data* p_data, UINT32 value);
*   \brief   Set the port A register.
*            
*            Writes the specified value to the port A register
*   \param   p_data Pointer to board data
*   \param   value The value to set
*   \return  TRUE: Procedure successfully executed
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1495_set_port_A_enable( cvt_V1495_data* p_data, UINT32 value);

////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1495_set_port_B_enable( cvt_V1495_data* p_data, UINT32 value);
*   \brief   Set the port B register.
*            
*            Writes the specified value to the port B register
*   \param   p_data Pointer to board data
*   \param   value The value to set
*   \return  TRUE: Procedure successfully executed
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1495_set_port_B_enable( cvt_V1495_data* p_data, UINT32 value);


////////////////////////////////////////////////////////////////////////////////////////////////
/*! \fn      BOOL cvt_V1495_set_port_D_enable( cvt_V1495_data* p_data, UINT32 value);
*   \brief   Set the port D register.
*            
*            Writes the specified value to the port D register
*   \param   p_data Pointer to board data
*   \param   value The value to set
*   \return  TRUE: Procedure successfully executed
*/
////////////////////////////////////////////////////////////////////////////////////////////////
BOOL cvt_V1495_set_port_D_enable( cvt_V1495_data* p_data, UINT32 value);

#endif 
